define([
	"dojo/_base/declare",
	"thornberry/layout/View",
	"dojo/_base/array",
	"thornberry/Toolbar",
	"thornberry/calendar/CalendarWidget",
	"thornberry/calendar/CalendarDetails",
	"thornberry/calendar/EventSummary",
	"thornberry/calendar/availabilityManager",
	"dojo/dom-construct",
	"dojo/dom-style",
	"dojo/on",
	"dojo/store/Memory",
	"dojo/store/Observable",
	"dijit/form/Button",
	"dojo/dom",
	"dojo/date",
	"thornberry/serverRequest",
	"dojo/date/locale",
	"dijit/form/DropDownButton",
	"dijit/DropDownMenu",
	"dijit/MenuItem",
	"dojo/_base/lang",
	"dojo/query",
	"dojo/json",
	"dojo/topic",
	"dojo/NodeList-dom"
], function (declare, View, arrayUtils, Toolbar, CalendarWidget,CalendarDetails,EventSummary,availabilityManager,domConstruct,domStyle,on,Memory,Observable,Button,dom,dateUtils,serverRequest,locale,DropDownButton,DropDownMenu,MenuItem,lang,query,JSON,topic) {
	/*
		Author: rwitherell
		Created: *Unknown*
		Purpose: 
		History:
		2022-08-30 WRA ND-12037 Added calendar reload icon.
	*/
	var CalendarView = declare([View], {
		id: "calendarView",
		title: "Calendar",
		calendar: null,
		calendarStore: null,

		actionButtons: null,

		patientCalendar: null,
		
		postMixInProperties: function() {
			var t = this;
			t.inherited(arguments);

			t.actionButtons = [];

			t.defer(function() {
				// Wait until the controller property has been set
				t.showList.push(t.controller.filterIconNode);
				t.showList.push(t.refreshIconNode);
				//t.showList.push(t.controller.printIconNode);
			});
		},

		buildContent: function() {
			var t = this;

			var previousButton = new Button({label: "◄"});
			var nextButton = new Button({label: "►"});

			var todayButton = new Button({
				label: "Today",
				style: "margin:0.2em"
			});
			
			var menu = new DropDownMenu();
			var dropDownButton = new DropDownButton({
				label: "Month",
				dropDown: menu
			});
			t.dropDownButton = dropDownButton;
			var menuOnClick = function() {
				var label = this.get("label");
				if (dropDownButton.get("label") !== label) {
					var startTimeOfDay = (new Date()).getHours();
					if (t.calendar.selectedItem) {
						var date = new Date(t.calendar.selectedItem._startTime);
						t.calendar.set("date",new Date(t.calendar.selectedItem._startTime));
						if (t.calendar.selectedItem._allDay) {
							startTimeOfDay = null;
						} else {
							startTimeOfDay = date.getHours();
						}
					}
					if (startTimeOfDay) {
						var hours = startTimeOfDay;
						t.defer(function() {
							hours = (hours >= 1 ? hours - 1 : hours);
							if (label === "Week" || label === "Day") {
								t.calendar.columnView._startTimeOfDayInvalidated = true;
								t.calendar.columnView.set("startTimeOfDay",{hours:hours});
							} else if (label === "Side-by-Side") {
								t.calendar.sideBySideView.set("startTimeOfDay",{hours:hours});
							}
						});
					}
				}
				dropDownButton.set("label",label);
			}
			var dayButton = new MenuItem({label: "Day", onClick: menuOnClick});
			var weekButton = new MenuItem({label: "Week", onClick: menuOnClick});
			var twoWeekButton = new MenuItem({label: "2 Weeks", onClick: menuOnClick});
			var monthButton = new MenuItem({label: "Month", onClick: menuOnClick});
			var certButton = new MenuItem({label: "Cert", onClick: menuOnClick, style:"display:none"});
			var agendaButton = new MenuItem({label: "Agenda", onClick: menuOnClick });
			var sideBySideButton = new MenuItem({label: "Side-by-Side", onClick: menuOnClick});

			t.dayButton = dayButton;
			t.certButton = certButton;
			t.monthButton = monthButton;
			t.sideBySideButton = sideBySideButton;

			var summaryButton = new Button({
				label: "Summary",
				onClick: function() {
					t.calendar.set("selectedItem",null);
					t.calendar.onChange();
					t.eventSummary.show();
				}
			});

			t.refreshIconNode = domConstruct.toDom('<div title="Reload Calendar" class="fas calendar-icon fa-redo icon" style="float: right; margin-top: 9px; margin-right: 3px; font-size: 20px;"></div>');
			t.controller.registerToggledNode(t.refreshIconNode);
			t.own(on(t.refreshIconNode,"click", function(){
				top.reloadCalendar(t.calendarDetails.getSelectedCalendars());	
			}))
		   
			menu.addChild(dayButton);
			menu.addChild(weekButton);
			menu.addChild(twoWeekButton);
			menu.addChild(monthButton);
			menu.addChild(t.certButton);
			menu.addChild(agendaButton);
			menu.addChild(sideBySideButton);
			dropDownButton.startup();

			t.calendar = new CalendarWidget({
				startTimeAttr: "_startTime",
				endTimeAttr: "_endTime",
				allDayAttr: "_allDay",
				summaryAttr: "_description",
				subColumnAttr: "_calendarId",
				date: new Date(),
				previousButton: previousButton,
				nextButton: nextButton,
				todayButton: todayButton,
				dayButton: dayButton,
				weekButton: weekButton,
				twoWeekButton: twoWeekButton,
				monthButton: monthButton,
				certButton: t.certButton,
				agendaButton: agendaButton,
				sideBySideButton: sideBySideButton,
				calendarStore: t.calendarStore,
				calendarView: t
			});
			domConstruct.place(t.calendar.domNode,t.domNode);

			domStyle.set(t.calendar.domNode,{
				top: "0px",
				position: "absolute",
				right: "25px",
				left: "0px",
				overflow: "auto",
				height: "100%"
			});

			t.calendarDetails = new CalendarDetails({
				calendarStore: t.calendarStore,
				calendar: t.calendar
			});
			t.calendarDetails.startup();
			domConstruct.place(t.calendarDetails.domNode,t.domNode);

			t.eventSummary = new EventSummary({
				calendar: t.calendar,
				calendarStore: t.calendarStore
			});
			t.eventSummary.startup();
			domConstruct.place(t.eventSummary.domNode,t.domNode);

			var centerContainer = domConstruct.toDom("<div id='centerActionContainer' style='flex: 0 1 auto;min-width:200px'></div>");
			centerContainer.appendChild(previousButton.domNode);
			centerContainer.appendChild(nextButton.domNode);
			t.dateRangeHeaderNode = domConstruct.toDom("<div id='dateRangeHeaderNode' class='workspace-title'></div>");
			centerContainer.appendChild(t.dateRangeHeaderNode);

			var actionContainer = domConstruct.toDom("<div id='actionContainer'></div>");
			t.controller.actionPane.domNode.appendChild(actionContainer);
			t.controller.registerToggledNode(actionContainer);
			t.showList.push(actionContainer);

			var leftContainer = domConstruct.toDom("<div id='leftActionContainer' style='flex: 1;display:flex;min-width:275px'></div>");
			actionContainer.appendChild(leftContainer);

			var addButton = new Button({
				label: "Add",
				onClick: function() {
					t.checkSyncOpen(lang.hitch(t,"onAdd"));
				},
				style: "margin-right:20px"
			});
			leftContainer.appendChild(addButton.domNode);

			t.viewButton = new Button({
				label: "View",
				onClick: function() {
					t.checkSyncOpen(function() {
						t.calendarDetails.show();
						t.eventSummary.hide();
						t.calendarDetails.closePatientEmployeeDetails();
					});
				}
			});

			leftContainer.appendChild(t.viewButton.domNode);
			t.controller.registerToggledNode(t.viewButton.domNode);

			t.actionContainer = domConstruct.create("div",{},leftContainer);
			domStyle.set(t.actionContainer,"display","inline-block");
			domStyle.set(t.actionContainer,"margin-right","20px");
			
			var toolbar = new Toolbar();
			t.toolbar = toolbar;
			leftContainer.appendChild(toolbar.domNode);
			toolbar.startup();
			domStyle.set(toolbar.domNode,"flex-grow","1");
			var toolbarActions = [];

			if (scheduleQueueAccess != 0) {
				toolbarActions.push({
					actionId: "scheduleQueue",
					type: "button",
					label: "Schedule Queue",
					onClick:function() {
						t.checkSyncOpen(function() {
							top.createScheduleQueueWorkspace();
						});
					}
				});
			}

			if (bulkEditAccess != 0) {
				toolbarActions.push({
					actionId: "bulkEdit",
					type: "button",
					label: "Bulk Edit",
					onClick: function() {
						t.checkSyncOpen(function() {
							var startDate = locale.format(t.calendar._timeInterval[0],{selector:"date",datePattern:"MM/dd/yyyy"});
							var endDate = locale.format(t.calendar._timeInterval[1],{selector:"date",datePattern:"MM/dd/yyyy"});
							var patient = controller.getPatient();
							var employees = controller.getEmployees();
							var users = arrayUtils.map(employees,function(employee) {
								var calendar = t.calendarStore.get(employee.id+"||Employee");
								return  {
									value: calendar.other.userId,
									label: employee.name
								}
							});
							top.createBulkEditWorkspace(startDate,endDate,JSON.stringify([patient]),JSON.stringify(users));
						});
					}
				});
			}

			toolbar.setActions(toolbarActions);

			actionContainer.appendChild(centerContainer);

			var rightContainer = domConstruct.toDom("<div id='rightActionContainer' style='text-align:right;flex: 1;text-align:center;min-width:225px'></div>");
			actionContainer.appendChild(rightContainer);

			rightContainer.appendChild(todayButton.domNode);
			rightContainer.appendChild(dropDownButton.domNode);
			rightContainer.appendChild(summaryButton.domNode);
			domConstruct.place(t.refreshIconNode, controller.helpIconNode, "after");

			t.calendar.startup();
		},

		postCreate: function() {
			var t = this;
			t.inherited(arguments);

			t.own(on(t.calendarDetails.miniCalendar,"change", function(){
				t.calendar.set("date",t.calendarDetails.miniCalendar.get("value"));
			}),
			on(t.calendarDetails,"changeSelectedCalendars",function(selectedIds) {
				// Sort by name and filter to only employees
				var calendars = t.calendarStore.query({type:"Employee"},{sort:[{attribute:"name"}]});
				var sideBySideList = [];
				arrayUtils.forEach(calendars,function(calendar) {
					if (arrayUtils.some(selectedIds,function(selectedId) {
						return selectedId === calendar.id;
					})) {
						sideBySideList.push(calendar.id);
					}
				});
				// Set the side-by-side view's set of calendars
				t.calendar.set("subColumns",sideBySideList);
				domStyle.set(t.sideBySideButton.domNode,"display",(sideBySideList.length > 0 ? "" : "none"));
				if (sideBySideList.length === 0 && t.dropDownButton.label === "Side-by-Side") {
					t.dayButton.onClick();
				}

				var patient = controller.getPatient();
				domStyle.set(t.certButton.domNode,"display",(patient ? "" : "none"));
				if (!patient && t.dropDownButton.label === "Cert") {
					t.monthButton.onClick();
				} else if (patient) {
					var patientCalendar = t.calendarStore.get(patient.value+"||Patient");
					if (t.dropDownButton.label === "Cert" && (!patientCalendar || patientCalendar.id !== t.patientCalendar.id)) {
						t.defer(function() {
							var currentDate = t.calendar.get("date") || t.calendar.get("startDate");
							var cert = patientCalendar.getCertByDate(currentDate);
							t.calendar.set("date",null);
							t.calendar.set("startDate",new Date(cert.start));
							t.calendar.set("endDate",new Date(cert.end));
						});
					}
					t.patientCalendar = patientCalendar;
				}
			}));
			t.calendarDetails.miniCalendar.set("value",t.calendar.get("date"));
			
			// configure item properties panel
			t.own(on(t.calendar,"timeIntervalChange",function(event){
				t.calendarDetails.miniCalendar.set("currentFocus", event.startTime, false);							
				t.calendarDetails.miniCalendar.set("minDate", event.startTime);
				t.calendarDetails.miniCalendar.set("maxDate", event.endTime);
				t.calendarDetails.miniCalendar._populateGrid();
				
				if (t.dropDownButton.label === "Cert") {
					var cert = t.patientCalendar.getCertByDate(event.startTime);
					var endDate = dateUtils.add(new Date(cert.end),"day",-1);
					t.dateRangeHeaderNode.innerHTML = "<span title='("+cert.start+" - "+locale.format(endDate,{selector:"date",datePattern:"MM/dd/yyyy"})+")'>Cert #"+cert.cert+"</span>";
				} else if (t.calendar.dateInterval === "month") {
					var format = {selector: "date", datePattern: "MMM yyyy"};
					t.dateRangeHeaderNode.innerHTML = locale.format(event.startTime,format);
				} else  {
					var format = {selector: "date", datePattern: "MMM. d"};
					t.dateRangeHeaderNode.innerHTML = locale.format(event.startTime,format)+(t.calendar.dateInterval === "day" ? "" : " - "+locale.format(dateUtils.add(event.endTime,"day",-1),format));
				}
			}));
			
			t.own(
				on(t.calendar,"itemEditEnd", function(event){
					event.item._item.event.propagateChanges(event.item,t.calendar,t.calendarStore);
				}),
				on(t.calendar,"change",function(e) {
					t.controller.refresh();
					if (e && e.newValue && e.newValue.event) {
						t.calendarDetails.showEventDetails(e.newValue.calendar,e.newValue.event);

						//t.calendar.set("date",new Date(e.newValue.event._startTime));
					} else {
						t.calendarDetails.closeEventDetails();
					}
				}),
				on(t.calendarDetails,"closeEventDetails",function() {
					t.calendar.set("selectedItem",null);
					t.calendar.onChange();
				},
				on(t.calendar,"gridDoubleClick",function(event) {
					var dateDefault = event.date;
					var timeDefault = null;
					if (event.sheet === "primary" && (t.dropDownButton.label === "Day" || t.dropDownButton.label === "Week" || t.dropDownButton.label === "Side-by-Side")) {
						event.date.setMinutes(Math.floor(event.date.getMinutes()/15)*15);
						timeDefault = event.date;
					}
					var event = {
						dateDefault: dateDefault,
						timeDefault: timeDefault,
						calendarDefault: event.calendar || null
					}
					t.onAdd(event);
				})
			));
		},

		setData: function(eventArray,okFunc) {
			var t = this;

			t.updateAvailability(okFunc);

			var calendarIndex = {};
			var availabilityCalendarId = null;
			arrayUtils.forEach(t.calendarDetails.getSelectedCalendars(),function(calendarId) {
				calendarIndex[calendarId] = true;
			});

			// Get an individual event for each calendar
			var calendarEventArray = [];
			arrayUtils.forEach(eventArray,function(event) {
				arrayUtils.forEach(event.getCalendarEvents(calendarIndex,t.calendarStore),function(calendarEvent) {calendarEventArray.push(calendarEvent)});
			});

			t.defer(function() {
				if (okFunc()) {
					var selectedItem = t.calendar.selectedItem;
					t.calendar.set("store",new Observable(new Memory({data:calendarEventArray})));

					t.eventSummary.updateStats();

					// Handle changes to the eventId
					if (selectedItem) {
						var newItem = t.calendar.store.get(selectedItem.id);
						if (newItem) {
							t.calendar.set("selectedItem",newItem);
						} else {
							t.calendar.set("selectedItem",null);
						}
						t.calendar.onChange({newValue:newItem});
					}
				}
			});
		},

		updateAvailability: function(okFunc) {
			var t = this;
			var label = t.dropDownButton.get("label");
			if (label === "Week" || label === "Day" || label === "Side-by-Side") {
				// wait until the time interval change has updated in the calendar
				t.defer(function() {
					var availabilityCalendarId = null;
					var calendarArray = [];
					arrayUtils.forEach(t.calendarDetails.getSelectedCalendars(),function(calendarId) {
						// Only gather availability if a single employee is currently selected
						var calendar = t.calendarStore.get(calendarId);
						if (calendar.type === "Employee") {
							if (label === "Side-by-Side") {
								calendarArray.push(calendar.id);
							} else {
								availabilityCalendarId = (availabilityCalendarId === null ? calendar.id : false);
							}
						}
					});

					var startTime = dateUtils.add((t.calendar._timeInterval ? t.calendar._timeInterval[0] : t.calendar.date),"day",-EXTRADAYS-1);
					var endTime = dateUtils.add((t.calendar._timeInterval ? t.calendar._timeInterval[1] : t.calendar.date),"day",EXTRADAYS+1);
					var availabilityArray = [];
					var calendarArray = (availabilityCalendarId ? [availabilityCalendarId] : calendarArray);
					availabilityManager.query(calendarArray,startTime,endTime).then(function(array) {
						if (okFunc()) {
							t.calendar.set("decorationStore",new Observable(new Memory({
								data: array || []
							})));
						}
					});
				});
			}
		},

		refresh: function() {
			var t = this;
			arrayUtils.forEach(t.actionButtons,function(button) {button.destroy();});

			if (t.calendar.selectedItems && t.calendar.selectedItems.length === 1){
				var item = t.calendar.selectedItems[0];
				domStyle.set(t.viewButton.domNode,"display","");

				var actions = item.event.getActions();
				arrayUtils.forEach(actions,function(action) {
					var button = new Button({
						label: action,
						onClick: function() {
							t.checkSyncOpen(function() {
								item.event.onAction(action);
							});
						}
					});
					t.actionButtons.push(button);
					t.actionContainer.appendChild(button.domNode);
				});
			}

			t.toolbar.resize();
			t.toolbar.resize(); // Two since it seems to be taking two calls to resize it properly
		},

		checkSyncOpen: function(callback) {
			if (typeof ISSERVER === "undefined" || !ISSERVER) {
				serverRequest("UI.Utility","RoutineOpen",{modules:["OPS"],routine:"SYNC"}).then(function(moduleName) {
					if (moduleName !== "") {
						actionShowMessage("The Synchronization process must be complete before selecting this option");
					} else {
						callback();
					};
				});
			} else {
				callback();
			}
		},

		onAdd: function(event) {}
	});

	return CalendarView;
});